//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "MimeMan.h"
#include "HeadMan.h"
#include "Global.h"
TMimeMan *MimeMan;
//---------------------------------------------------------------------------
bool __fastcall AttachFiles(TStrings *Message, TStrings *Display,
   TProgressBar *ProgressBar)
{
   MimeMan = new TMimeMan(Message,Display,ProgressBar);
   MimeMan->AttachFiles();
   bool ok = MimeMan->status==statMimeOk;
   delete MimeMan; MimeMan = NULL;
   return ok;
}
//---------------------------------------------------------------------------
__fastcall TMimeMan::TMimeMan(TStrings *message, TStrings *display,
   TProgressBar *progressbar)
{
   Filenames = new TStringList();
   ProgressBar = progressbar;
   ProgressBar->Position = 0;
   ProgressBar->Max = 100;
   ProgressBar->Min = 0;
   status = statMimeOk;
   Message = message;
   Display = display;
   Cancelled = false;
}
//---------------------------------------------------------------------------
__fastcall TMimeMan::~TMimeMan()
{
   delete Filenames;
}
//---------------------------------------------------------------------------
int __fastcall TMimeMan::EOH()
{
   int eoh = Message->Count-1;
   for (int i=0; i<Message->Count; i++)
      if (Message->Strings[i].Trim().IsEmpty())
      {
         eoh = i;
         break;
      }
   if (eoh==Message->Count-1)
      Message->Add("");
   return eoh;
}
//---------------------------------------------------------------------------
int __fastcall TMimeMan::GetAttachmentNames()
{
   int count=0;
   AnsiString fname,line;
   while ((line = HeadMan->GetDat("Attachment: ",Message)) != "")
   {
      HeadMan->Delete("Attachment: ");
      int i=0; while ((fname=GetParam(i,line,",").LowerCase().Trim())!="")
      {
         // remove quotes if necessary -- if one at begining assume both ends
         if (fname.SubString(1,1)=="\"") fname = fname.SubString(2,fname.Length()-2);
         if (!FileExists(fname))
         {
            if (MsgBox("The file attachment \""+fname+"\" does not exist!\n\n Continue without?",
               "Attachment Error",MB_YESNO)==IDNO)
            {
               status = statMimeBadFilename;
               break;
            }
         }
         Filenames->Add(fname); count++;
         i++;
      }
   }
   return count;
}
//---------------------------------------------------------------------------
bool __fastcall TMimeMan::AttachFiles()
{
   if (!GetAttachmentNames() || status==statMimeBadFilename) return false;

   int pos;
   MimeBoundary = "= Multipart Boundary " + FormatDateTime("mmddyyhhnn", Now());
   HeadMan->Delete("MIME-Version: ");
   HeadMan->Delete("Content-Type: ");
   Message->Insert(EOH(),"MIME-Version: 1.0");
   Message->Insert(EOH(),"Content-Type: multipart/mixed;");
   Message->Insert(EOH(),"\tboundary=\""+MimeBoundary+"\"");
   pos = EOH()+1;
   Message->Insert(pos++,"This is a multipart MIME message.");
   Message->Insert(pos++,"");
   Message->Insert(pos++,"--" + MimeBoundary);
   Message->Insert(pos++,"Content-Type: text/plain; charset=\"iso-8859-1\"");
   Message->Insert(pos++,"Content-Transfer-Encoding: 7bit");
   Message->Insert(pos++,"");

   AnsiString fname;
   for (int i=0; i<Filenames->Count; i++)
   {
      fname = Filenames->Strings[i];
      if (!FileExists(fname)) continue;
      //if (status==statMimeBadFilename)
      //{
      //   Display->Add("\r\n Error: "+fname+" does not exist!");
      //   //Display->Add(" Continuing with next email.");
      //   break;
      //}
      if (Cancelled) {
         status = statMimeCancel;
         break;
      }
      AttachFile(fname);
   }
   if (status==statMimeOk)
   {
      Message->Add("");
      Message->Add("--"+MimeBoundary+"--");
   }
   return status==statMimeOk;
}
//---------------------------------------------------------------------------
void __fastcall TMimeMan::AttachFile(TFileName &Filename)
{
   while (Message->Count && Message->Strings[Message->Count-1].IsEmpty())
      Message->Delete(Message->Count-1);
   Message->Add("");
   AnsiString FileName    = ExtractFileName(Filename);
   Message->Add("--" + MimeBoundary);
   Message->Add("Content-Type: "+GetContentType(FileName)+"; name=\""+FileName+"\"");
   Message->Add("Content-Transfer-Encoding: base64");
   Message->Add("Content-Disposition: attachment; filename=\""+FileName+"\"");
   Message->Add("");

   UUEncode(Filename);
}
//---------------------------------------------------------------------------
void __fastcall TMimeMan::UUEncode(AnsiString &Filename)
{
   const char Base64Chars[] =
   {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   char DataIn[3];
   char DataOut[81];
   ProgressBar->Position = 0;
   ProgressBar->Visible = true;
   Display->Add(" Mime encoding file: \""+Filename+"\"");
   int i,Count=0,BytesRd,InBytes=sizeof(DataIn);
   TFileStream *f = new TFileStream(Filename,fmOpenRead);
   if (!f) return;

   while (true)
   {
      BytesRd=f->Read(DataIn,InBytes);
      for (i=1; i<4; i++) DataOut[Count+i]=0x40;

      double p=f->Position,s=f->Size,pos=(p/s)*100;
      ProgressBar->Position = (WORD)pos; Application->ProcessMessages();

      DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
      DataOut[Count+1] = char((DataIn[0] & 0x03) << 4);
      if (BytesRd>1)
      {
         DataOut[Count+1] += char((DataIn[1] & 0xF0) >> 4);
         DataOut[Count+2]  = char((DataIn[1] & 0x0F) << 2);
         if (BytesRd>2)
         {
            DataOut[Count+2] += char((DataIn[2] & 0xC0) >> 6);
            DataOut[Count+3]  =  char(DataIn[2] & 0x3F);
         }
      }

      for (i=0; i<4; i++) DataOut[Count+i] = byte(Base64Chars[DataOut[Count+i]]);

      Count += 4; if (Count<60 && BytesRd==InBytes) continue;

      DataOut[Count] = '\0';
      Message->Add(AnsiString(&DataOut[0]));

      if (Cancelled) {
         status = statMimeCancel;
         break;
      }

      if (BytesRd<InBytes) break;

      Count=0;
   }
   ProgressBar->Visible = false;
   delete f;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMimeMan::GetContentType(AnsiString &Filename)
{
  AnsiString type,Ext = ExtractFileExt(Filename).UpperCase();
  if (Ext==".HTM" || Ext==".HTML") type = "text/html"; else
  if (Ext==".AU"  || Ext==".SND" ) type = "audio/basic"; else
  if (Ext==".AIF") type = "audio/aiff"; else
  if (Ext==".GIF") type = "image/gif"; else
  if (Ext==".JPG") type = "image/jpeg"; else
  if (Ext==".AVI") type = "video/avi"; else
  if (Ext==".RTF") type = "text/rtf"; else
  if (Ext==".TXT") type = "text/plain"; else
     type = "application/octet-stream";
  return type;
}
//---------------------------------------------------------------------------
/*AnsiString __fastcall Base64EncodeLine(AnsiString &instr)
// used in SMTP password authentication
{
   AnsiString outstr;
   const char Base64Chars[] =
   {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   char DataIn[3];
   char DataOut[81];
   int i,Count=0,BytesRd,InBytes=sizeof(DataIn);
   TMemoryStream *f = new TMemoryStream();
   if (!f) return;

   f->Write(instr.c_str(),instr.Length());
   f->Seek(0,soFromBeginning);
   while (true)
   {
      BytesRd=f->Read(DataIn,InBytes);
      for (i=1; i<4; i++) DataOut[Count+i]=0x40;

      DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
      DataOut[Count+1] = char((DataIn[0] & 0x03) << 4);
      if (BytesRd>1)
      {
         DataOut[Count+1] += char((DataIn[1] & 0xF0) >> 4);
         DataOut[Count+2]  = char((DataIn[1] & 0x0F) << 2);
         if (BytesRd>2)
         {
            DataOut[Count+2] += char((DataIn[2] & 0xC0) >> 6);
            DataOut[Count+3]  =  char(DataIn[2] & 0x3F);
         }
      }

      for (i=0; i<4; i++) DataOut[Count+i] = byte(Base64Chars[DataOut[Count+i]]);

      Count += 4; if (Count<120 && BytesRd==InBytes) continue;

      DataOut[Count] = '\0';
      outstr += &DataOut[0];

      if (BytesRd<InBytes) break;

      Count=0;
   }
   instr = outstr;
   delete f;
   return instr;
}*/
//---------------------------------------------------------------------------

